module Main (run, main) where

import IO
import Maybe
import List
import Parser
import System

showError = putStrLn . ioeGetErrorString

main :: IO ()
main =
  do { args <- getArgs
     ; if (length args /= 2) 
          then fail "Program requires two command line parameters"
          else run (args!!0) (args!!1)
     } `catch` showError


run filename1 filename2 = 
  do { ps1 <- readPieces filename1
     ; ps2 <- readPieces filename2
     ; match ps1 ps2
     ; let (x,y) = measure ps2  
     ; putStrLn ("Smallest rectangle size "++show x++"x"++show y++" (area is "++show (x*y)++")")
     }

measure :: [Piece] -> (Int, Int)
measure ps = (maximum xs - minimum xs + 1, maximum ys - minimum ys + 1)
  where
    (xs,ys) = unzip $ concat $ map (snd . piece) ps


ps1 `match` ps2
  | not(null d1) = fail ("Error: Piece "++head d1++" appears in input but not in output file")
  | not(null d2) = fail ("Error: Piece "++head d2++" appears in output but not in input file")
  | not(null ov) = fail ("Error: Pieces "++(fst $ head ov)++" and "++(snd $ head ov)++" overlap")
  | not(null bd) = fail ("Error: Piece "++head bd++" in the output does not match the one given in the input")
  | otherwise    = return ()
  where
    ps1' = map piece ps1
    ps2' = map piece ps2
 
    ns1 = map fst ps1'
    ns2 = map fst ps2'
  
    d1  = ns1 \\ ns2
    d2  = ns2 \\ ns1

    ps  = [ (n,p1,fromJust (lookup n ps2')) | (n,p1) <- ps1' ]
    bd  = [ n | (n,p1,p2) <- ps, not (p1 === p2) ]

    ov  = [ (n,n') | ((n,p):rest) <- tails ps2', (n',p') <- rest, p `overlaps` p']


(===) :: [Pos] -> [Pos] -> Bool
poss1 === poss2 = or [p1 `sameAs` p2 | p1 <- possibilities1, p2 <- possibilities2 ]
  -- assumes poss1 starts with (0,0)
  where
    rot90 (x,y) = (-y,x)
    possibilities1 = 
      let poss1'   = map rot90 poss1   
          poss1''  = map rot90 poss1' 
          poss1''' = map rot90 poss1''
      in  [poss1,poss1',poss1'',poss1''']

    trs (dx,dy) (x,y) = (x-dx, y-dy)
    possibilities2 = 
      [ map (trs pos) poss2 | pos <- poss2 ]

    ps1 `sameAs` ps2 = 
      ps1 `subseteq` ps2 && ps2 `subseteq` ps1


s1 `subseteq` s2 = all (`elem` s2) s1
l1 `overlaps` l2 = any (`elem` l2) l1





