module Main (run, main) where

import IO
import Maybe
import List
import Parser
import System

showError = putStrLn . ioeGetErrorString

main :: IO ()
main =
  do { args <- getArgs
     ; if (length args /= 2) 
          then fail "Program requires two command line parameters: piecelist filename and ps filename"
          else run (args!!0) (args!!1)
     } `catch` showError


xmargin = 72*1
ymargin = 72*1
width   = 72*6
height  = 72*10

run filename_pcs filename_ps= 
  do { ps <- readPieces filename_pcs
     ; ok ps
     ; let ((xbl,ybl),(xtr,ytr)) = limits ps
     ; let header = "/ss "++show (min (width `div` (xtr-xbl+1)) (height `div` (ytr-ybl+1)))++ " def\n"++
                    "/square { /y exch def /x exch def x ss mul "++show xmargin++" add y ss mul "++show ymargin++" add moveto\n"++
                    "          gsave ss 0 rlineto 0 ss rlineto ss neg 0 rlineto closepath\n"++
                    "          gsave r g b setrgbcolor fill grestore 0 0 0 setrgbcolor stroke \n"++
                    "          grestore ss 2 div dup rmoveto show } def \n"++
                    "/Times-Roman findfont ss 5 div scalefont setfont\n"
     ; let footer = "\n\nshowpage\n"
     ; writeFile filename_ps (header ++ toPS ps (xbl,ybl) ++ footer)
     }


toPS ps (xbl,ybl) = drawPieces colours pieces
  where
    colours = [(1,1,1),(1,0,0),(0,1,0),(0,0,1),(1,1,0),(1,0,1),(0,1,1),(0.5,1,1),(1,0.5,1),(1,1,0.5)
              ,(0.5,0,0),(0,0.5,0),(0,0,0.5),(1,0,0.5),(0,1,0.5),(0.5,1,0),(0.5,0,1),(0,0.5,1),(1,0.5,0)
              ,(0.5,0.5,0.5),(0.5,0.5,0),(0.5,0,0.5),(0,0.5,0.5),(0.5,0.5,1),(0.5,1,0.5),(1,0.5,0.5)
              ]++repeat (1,1,1)
    pieces = map piece ps

    drawPiece (r,g,b) (n,sqs) = unlines 
                                [ "/r "++show r++" def /g "++show g++" def /b "++show b++" def "++
                                  "("++n++") "++show (x-xbl)++" "++show (y-ybl)++" square" | (x,y) <- sqs ]

    drawPieces _ [] = ""
    drawPieces (c:cs) (s:ss) = drawPiece c s++"\n"++drawPieces cs ss

limits ps = ((minimum xs, minimum ys), (maximum xs, maximum ys))
  where
    (xs,ys) = unzip $ concat $ map (snd . piece) ps

ok ps
  | not(null ov) = fail ("Error: Pieces "++(fst $ head ov)++" and "++(snd $ head ov)++" overlap")
  | otherwise    = return ()
  where
    ps' = map piece ps
    ns  = map fst ps'
    ov  = [ (n,n') | ((n,p):rest) <- tails ps', (n',p') <- rest, p `overlaps` p']

l1 `overlaps` l2 = any (`elem` l2) l1

